Package: bluster
Version: 1.22.0
Date: 2026-03-05
Title: Clustering Algorithms for Bioconductor
Description: Wraps common clustering algorithms in an easily extended
        S4 framework. Backends are implemented for hierarchical,
        k-means and graph-based clustering. Several utilities are also
        provided to compare and evaluate clustering results.
Authors@R: 
    c(
        person("Aaron", "Lun", role = c("aut", "cre"), email = "infinite.monkeys.with.keyboards@gmail.com"),
        person("Stephanie", "Hicks", role="ctb"),
        person("Basil", "Courbayre", role="ctb"),
        person("Tuomas", "Borman", role="ctb"),
        person("Leo", "Lahti", role="ctb")
    )
Imports: stats, methods, utils, cluster, Matrix, Rcpp, igraph,
        S4Vectors, BiocParallel, BiocNeighbors
Suggests: knitr, rmarkdown, testthat, BiocStyle, dynamicTreeCut,
        scRNAseq, scuttle, scater, scran, pheatmap, viridis, mbkmeans,
        kohonen, apcluster, DirichletMultinomial, vegan, fastcluster
biocViews: ImmunoOncology, Software, GeneExpression, Transcriptomics,
        SingleCell, Clustering
LinkingTo: Rcpp, assorthead
Collate: AllClasses.R AllGenerics.R AgnesParam.R approxSilhouette.R
        bluster-package.R DbscanParam.R DianaParam.R AffinityParam.R
        BlusterParam.R bootstrapStability.R ClaraParam.R clusterRMSD.R
        clusterSweep.R compareClusterings.R DmmParam.R
        FixedNumberParam.R HclustParam.R HierarchicalParam.R
        KmeansParam.R linkClusters.R makeSNNGraph.R MbkmeansParam.R
        mergeCommunities.R neighborPurity.R nestedClusters.R
        NNGraphParam.R pairwiseModularity.R pairwiseRand.R PamParam.R
        RcppExports.R SomParam.R TwoStepParam.R utils.R
License: GPL-3
NeedsCompilation: yes
VignetteBuilder: knitr
SystemRequirements: C++17
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/pak/sysreqs: libglpk-dev libxml2-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:53:10 UTC
RemoteUrl: https://github.com/bioc/bluster
RemoteRef: RELEASE_3_23
RemoteSha: c564e528da63e928b8a22e01bf80c148ce8042a3
Packaged: 2026-04-29 15:00:03 UTC; root
Author: Aaron Lun [aut, cre],
  Stephanie Hicks [ctb],
  Basil Courbayre [ctb],
  Tuomas Borman [ctb],
  Leo Lahti [ctb]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 15:06:31 UTC; unix
