# Generated by roxygen2: do not edit by hand

export("centers<-")
export(.defaultScalarArguments)
export(.extractScalarArguments)
export(.showScalarArguments)
export(AffinityParam)
export(AgnesParam)
export(ClaraParam)
export(DbscanParam)
export(DianaParam)
export(DmmParam)
export(HclustParam)
export(KNNGraphParam)
export(KmeansParam)
export(MbkmeansParam)
export(NNGraphParam)
export(PamParam)
export(SNNGraphParam)
export(SomParam)
export(TwoStepParam)
export(approxSilhouette)
export(bootstrapStability)
export(centers)
export(clusterRMSD)
export(clusterRows)
export(clusterSweep)
export(compareClusterings)
export(linkClusters)
export(linkClustersMatrix)
export(makeKNNGraph)
export(makeSNNGraph)
export(mergeCommunities)
export(neighborPurity)
export(neighborsToKNNGraph)
export(neighborsToSNNGraph)
export(nestedClusters)
export(pairwiseModularity)
export(pairwiseRand)
exportClasses(AffinityParam)
exportClasses(AgnesParam)
exportClasses(BlusterParam)
exportClasses(ClaraParam)
exportClasses(DbscanParam)
exportClasses(DianaParam)
exportClasses(DmmParam)
exportClasses(FixedNumberParam)
exportClasses(HclustParam)
exportClasses(HierarchicalParam)
exportClasses(KNNGraphParam)
exportClasses(KmeansParam)
exportClasses(MbkmeansParam)
exportClasses(NNGraphParam)
exportClasses(PamParam)
exportClasses(SNNGraphParam)
exportClasses(SomParam)
exportClasses(TwoStepParam)
exportMethods("[[")
exportMethods("[[<-")
exportMethods("centers<-")
exportMethods(.defaultScalarArguments)
exportMethods(centers)
exportMethods(clusterRows)
exportMethods(show)
exportMethods(updateObject)
import(methods)
importFrom(BiocNeighbors,KmknnParam)
importFrom(BiocNeighbors,buildIndex)
importFrom(BiocNeighbors,findDistance)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocNeighbors,findNeighbors)
importFrom(BiocNeighbors,queryKNN)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpisup)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(Matrix,"diag<-")
importFrom(Matrix,diag)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,I)
importFrom(S4Vectors,List)
importFrom(S4Vectors,coolcat)
importFrom(S4Vectors,setValidity2)
importFrom(S4Vectors,updateObject)
importFrom(cluster,agnes)
importFrom(cluster,clara)
importFrom(cluster,diana)
importFrom(cluster,pam)
importFrom(igraph,"E<-")
importFrom(igraph,E)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,components)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,is.directed)
importFrom(igraph,make_graph)
importFrom(igraph,membership)
importFrom(igraph,modularity)
importFrom(igraph,simplify)
importFrom(stats,as.hclust)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(bluster)
