# Clustering methods for Bioconductor

|Build|Status|
|-----|----|
| Bioc-release | [![](http://bioconductor.org/shields/build/release/bioc/bluster.svg)](http://bioconductor.org/checkResults/release/bioc-LATEST/bluster) |
| Bioc-devel   | [![](http://bioconductor.org/shields/build/devel/bioc/bluster.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/bluster) | 

This package provides a consistent interface to a variety of clustering algorithms for convenient use in the rest of the Bioconductor ecosystem.
Check out the user guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/bluster.html) for more details.
