# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

build_snn_rank <- function(neighbors) {
    .Call('_bluster_build_snn_rank', PACKAGE = 'bluster', neighbors)
}

build_snn_number <- function(neighbors) {
    .Call('_bluster_build_snn_number', PACKAGE = 'bluster', neighbors)
}

sum_neighbor_weights <- function(nclusters, neighbors, clusters, weights) {
    .Call('_bluster_sum_neighbor_weights', PACKAGE = 'bluster', nclusters, neighbors, clusters, weights)
}

